@@ -3,6 +3,7 @@
 #include <locale.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gio/gio.h>
 
 #include <string.h>
@@ -97,6 +98,12 @@ test_resolution (void)
 		return;
 	}
 
+	if (g_strcmp0 (g_content_type_guess ("foo.html", NULL, 0, NULL), "text/html") != 0) {
+		g_test_message ("“foo.html” isn't detected as text/html, is shared-mime-info installed?");
+		g_test_fail ();
+		return;
+	}
+
 	/* http://bugzilla.gnome.org/show_bug.cgi?id=555417 */
 	g_assert_cmpstr (test_resolution_real ("http://www.yle.fi/player/player.jsp", "288629.asx?s=1000"), ==, "http://www.yle.fi/player/288629.asx?s=1000");
 	g_assert_cmpstr (test_resolution_real ("http://www.yle.fi/player/player.jsp?actionpage=3&id=288629&locale", "288629.asx?s=1000"), ==, "http://www.yle.fi/player/288629.asx?s=1000");
@@ -193,7 +200,6 @@ test_data_get_data (const char *uri, guint *len)
 	return buffer;
 }
 
-#ifdef HAVE_QUVI
 static void
 test_videosite (void)
 {
@@ -202,7 +208,6 @@ test_videosite (void)
 	g_test_message ("Testing data parsing \"%s\"...", uri);
 	g_assert_true (totem_pl_parser_can_parse_from_uri (uri, TRUE));
 }
-#endif
 
 static void
 test_parsability (void)
@@ -501,35 +506,28 @@ parser_test_get_playlist_field (const char *uri,
 static void
 test_image_link (void)
 {
-#ifdef HAVE_QUVI
 	char *uri;
 
 	/* From http://www.101greatgoals.com/feed/ */
 	uri = get_relative_uri (TEST_SRCDIR "empty-feed.xml");
 	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, NULL);
 	g_free (uri);
-#endif
 }
 
 static void
 test_no_url_podcast (void)
 {
-#ifdef HAVE_QUVI
 	char *uri;
 
 	/* From http://feeds.guardian.co.uk/theguardian/football/rss */
 	uri = get_relative_uri (TEST_SRCDIR "no-url-podcast.xml");
 	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, "http://www.guardian.co.uk/sport/video/2012/jul/26/london-2012-north-korea-flag-video");
 	g_free (uri);
-#endif
 }
 
-#ifdef HAVE_QUVI
 static void
 test_youtube_starttime (void)
 {
-/* Those do not work with quvi */
-#if 0
 	const char *uri;
 
 	/* old type of direct link */
@@ -543,9 +541,7 @@ test_youtube_starttime (void)
 	/* new type of direct link */
 	uri = "http://www.youtube.com/watch?v=Fk2bUvrv-Uc&t=2m30s";
 	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_STARTTIME), ==, "150");
-#endif
 }
-#endif /* HAVE_QUVI */
 
 static void
 test_itms_parsing (void)
@@ -644,6 +640,15 @@ test_parsing_xspf_escaping (void)
 	g_free (uri);
 }
 
+static void
+test_parsing_xspf_metadata (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "playlist.xspf");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_TITLE), ==, "Test Playlist");
+	g_free (uri);
+}
+
 static void
 test_parsing_xspf_xml_base (void)
 {
@@ -733,12 +738,76 @@ test_parsing_rtsp_text (void)
 }
 
 static void
-test_parsing_content_type (void)
+test_parsing_needle_carriage_return (void)
 {
 	char *uri;
+
+	/* rss needle test */
+	uri = get_relative_uri (TEST_SRCDIR "rss-needle-carriage-return");
+	g_assert_cmpuint (parser_test_get_num_entries (uri), ==, 19);
+	g_free (uri);
+
+	/* atom needle test */
+	uri = get_relative_uri (TEST_SRCDIR "atom.xml");
+	g_assert_cmpuint (parser_test_get_num_entries (uri), ==, 1);
+	g_free (uri);
+
+	/* opml needle test */
+	uri = get_relative_uri (TEST_SRCDIR "feeds.opml");
+	g_assert_cmpuint (parser_test_get_num_entries (uri), ==, 7);
+	g_free (uri);
+}
+
+static void
+test_parsing_feed_content_type (void)
+{
+	char *uri;
+
+	/* rss */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-description.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE), ==, "application/rss+xml");
+	g_free (uri);
+
+	/* atom */
+	uri = get_relative_uri (TEST_SRCDIR "atom.xml");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE), ==, "application/atom+xml");
+	g_free (uri);
+
+	/* opml */
+	uri = get_relative_uri (TEST_SRCDIR "feeds.opml");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE), ==, "text/x-opml+xml");
+	g_free (uri);
+}
+
+static void
+test_parsing_item_content_type (void)
+{
+	char *uri;
+
+	/* no audio content */
+	uri = get_relative_uri (TEST_SRCDIR "no-url-podcast.xml");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE), ==, NULL);
+	g_free (uri);
+
+	/* <enclosure> without <media:content> */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-description.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE), ==, "audio/mpeg");
+	g_free (uri);
+
+	/* <media:content> followed by <enclosure> */
 	uri = get_relative_uri (TEST_SRCDIR "HackerMedley");
 	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE), ==, "audio/mpeg");
 	g_free (uri);
+
+	/* <enclosure> followed by <media:content> */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-image-url.2.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE), ==, "audio/mpeg");
+	g_free (uri);
+
+	/* <enclosure> followed by <media:content> with image */
+	uri = get_relative_uri (TEST_SRCDIR "791154-kqed.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE), ==, "audio/mpeg");
+	g_free (uri);
 }
 
 static void
@@ -750,6 +819,271 @@ test_parsing_medium (void)
 	g_free (uri);
 }
 
+static void
+test_parsing_feed_description (void)
+{
+	char *uri;
+	const char *description1;
+	const char *description2;
+
+	description1 =
+		"At the end of the day, we're all black. Can we just get along? A podcast featuring "
+		"conversations to improve understanding between Africans and African Americans.";
+	description2 =
+		"Bastian Bielendorfer und Reinhard Remfort haben sich vor einigen Jahren bei einem "
+		"Dreh für das ZDF kennengelernt und schnell rausgefunden, dass sie eine gemeinsame "
+		"Vergangenheit teilen. Beide kommen aus dem tiefsten Ruhrpott, beide waren die dicken "
+		"Kinder in der Klasse und beide haben mindestens ein \"Sachbuch\" geschrieben. Heute "
+		"stehen die beiden vor der Kamera, auf Bühnen und sprechen in Mikrofone um Wissen zu "
+		"verbreiten und den Menschen in Ihrer Umgebung ein Lächeln ins Gesicht zu zaubern.\n\n"
+		"\"Alliteration am Arsch\" ist dabei der Versuch die Menschen an dem Leben zweier "
+		"ehemals dicker Kinder aus den 80ern teilhaben zu lassen.";
+
+	/* test for longer feed description */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-description.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_DESCRIPTION), ==, description1);
+	g_free (uri);
+
+	/* test for empty feed description tags */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-empty-description.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_DESCRIPTION), ==, description2);
+	g_free (uri);
+}
+
+static void
+test_parsing_item_description (void)
+{
+	char *uri;
+	const char *description1;
+	const char *description2;
+
+	description1 =
+		"H&M ad and hair standards in the black community";
+	description2 =
+		"Wie versprochen die zweite Hälfte unseres kleinen Auftritts in Frankfurt. Ist ab sofort auch in voller Länge auf YouTube zu finden: https://youtu.be/GAQakfNHGj8";
+
+	/* test for longer item description */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-description.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_DESCRIPTION), ==, description1);
+	g_free (uri);
+
+	/* test for empty item description */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-empty-description.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_DESCRIPTION), ==, description2);
+	g_free (uri);
+}
+
+static void
+test_parsing_feed_image (void)
+{
+	char *uri;
+
+	/* <itunes:image> followed by <image> */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-image-url.1.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_IMAGE_URI), ==, "http://i1.sndcdn.com/avatars-000325311522-dw14t0-original.jpg");
+	g_free (uri);
+
+	/* <image> followed by <itunes:image> */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-image-url.2.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_IMAGE_URI), ==, "http://ichef.bbci.co.uk/images/ic/3000x3000/p076j2sr.jpg");
+	g_free (uri);
+}
+
+static void
+test_parsing_item_image (void)
+{
+	char *uri;
+
+	/* no podcast item image */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-image-url.2.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_IMAGE_URI), ==, NULL);
+	g_free (uri);
+
+	/* only <itunes:image> */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-image-url.1.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_IMAGE_URI), ==, "http://i1.sndcdn.com/avatars-000325311522-dw14t0-original.jpg");
+	g_free (uri);
+
+	/* <itunes:image> followed by <media:content> with image */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-different-item-images.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_IMAGE_URI), ==, "https://images.theabcdn.com/i/37623804.jpg");
+	g_free (uri);
+
+	/* <image><url> followed by <itunes:image> */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-empty-description.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_IMAGE_URI), ==, "https://images.podigee.com/0x,svlSD5_BDle5m2pLyGQT7_GWO0GW8iv2Kgr6AhbFe8vU=/https://cdn.podigee.com/uploads/u2254/dafcf335-4257-4401-bdc3-349ef792aba4.jpg");
+	g_free (uri);
+}
+
+static void
+test_parsing_feed_pubdate (void)
+{
+	char *uri;
+
+	/* no <lastBuildDate> or <pubDate> */
+	uri = get_relative_uri (TEST_SRCDIR "585407.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_PUB_DATE), ==, NULL);
+	g_free (uri);
+
+	/* only <lastBuildDate> */
+	uri = get_relative_uri (TEST_SRCDIR "791154-kqed.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_PUB_DATE), ==, "Mon, 04 Dec 2017 08:01:09 +0000");
+	g_free (uri);
+
+	/* same <lastBuildDate> and <pubDate> */
+	uri = get_relative_uri (TEST_SRCDIR "560051.xml");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_PUB_DATE), ==, "Mon, 8 Dec 2008 13:20:00 CST");
+	g_free (uri);
+
+	/* <pubDate> followed by <lastBuildDate> */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-empty-description.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_PUB_DATE), ==, "Sun, 26 Jul 2020 20:07:40 +0000");
+	g_free (uri);
+
+	/* <lastBuildDate> followed by <pubDate> */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-image-url.1.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_PUB_DATE), ==, "Wed, 23 Aug 2017 01:55:17 +0000");
+	g_free (uri);
+
+}
+
+static void
+test_parsing_feed_author (void)
+{
+	char *uri;
+
+	/* no <itunes:owner> or <itunes:author> */
+	uri = get_relative_uri (TEST_SRCDIR "541405.xml");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_AUTHOR), ==, NULL);
+	g_free (uri);
+
+	/* <itunes:owner> without <itunes:author> */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-empty-description.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_AUTHOR), ==, "Bastian Bielendorfer und Reinhard Remfort");
+	g_free (uri);
+
+	/* same <itunes:owner> and <itunes:author> */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-image-url.1.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_AUTHOR), ==, "Exit Poll New England");
+	g_free (uri);
+
+	/* different <itunes:owner> with <itunes:author> */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-image-url.2.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_AUTHOR), ==, "BBC Radio");
+	g_free (uri);
+}
+
+static void
+test_parsing_feed_explicit (void)
+{
+	char *uri;
+
+	/* clean feed */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-image-url.1.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_RATING), ==, TOTEM_PL_PARSER_CONTENT_RATING_CLEAN);
+	g_free (uri);
+
+	/* explicit feed */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-empty-description.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_RATING), ==, TOTEM_PL_PARSER_CONTENT_RATING_EXPLICIT);
+	g_free (uri);
+
+	/* unrated feed */
+	uri = get_relative_uri (TEST_SRCDIR "content-no-rating.rss");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_RATING), ==, TOTEM_PL_PARSER_CONTENT_RATING_UNRATED);
+	g_free (uri);
+}
+
+static void
+test_parsing_item_explicit (void)
+{
+	char *uri;
+
+	/* clean item */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-image-url.1.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_RATING), ==, TOTEM_PL_PARSER_CONTENT_RATING_CLEAN);
+	g_free (uri);
+
+	/* explicit item */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-empty-description.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_RATING), ==, TOTEM_PL_PARSER_CONTENT_RATING_EXPLICIT);
+	g_free (uri);
+
+	/* unrated item */
+	uri = get_relative_uri (TEST_SRCDIR "content-no-rating.rss");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_RATING), ==, TOTEM_PL_PARSER_CONTENT_RATING_UNRATED);
+	g_free (uri);
+}
+
+static void
+test_invalid_characters (void)
+{
+	char *uri;
+
+	uri = get_relative_uri (TEST_SRCDIR "invalid-characters.rss");
+#ifdef HAVE_UCHARDET
+	if (g_test_subprocess ()) {
+		/* This call should abort with 'Invalid byte sequence in conversion input' */
+		simple_parser_test (uri);
+		g_assert_not_reached ();
+		return;
+	}
+
+	g_test_trap_subprocess (NULL, 0, 0);
+	g_test_trap_assert_failed ();
+	g_test_trap_assert_stderr ("*byte offset 22493,*");
+#else
+	g_assert_cmpint (simple_parser_test (uri), !=, TOTEM_PL_PARSER_RESULT_SUCCESS);
+#endif
+	g_free (uri);
+}
+
+static void
+test_invalid_utf8_characters (void)
+{
+	char *uri;
+
+	/* Test all entries have been parsed by checking entry count */
+	uri = get_relative_uri (TEST_SRCDIR "invalid-utf8-characters.rss");
+#ifdef HAVE_UCHARDET
+	g_assert_cmpuint (parser_test_get_num_entries (uri), ==, 4);
+#else
+	g_assert_cmpint (simple_parser_test (uri), !=, TOTEM_PL_PARSER_RESULT_SUCCESS);
+#endif
+	g_free (uri);
+}
+
+static void
+test_parsing_xml_cdata (void)
+{
+	char *uri;
+	const char *description =
+		"POLL (Podcast LPM LONTAR) merupakan sebuah media yang kami gunakan "
+		"untuk menyalurkan potensi anggota LPM LONTAR dengan menyajikan beragam topik diskusi.";
+
+	uri = get_relative_uri (TEST_SRCDIR "cdata.rss");
+
+	/* empty cdata */
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_COPYRIGHT), ==, NULL);
+
+	/* single char */
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_CONTACT), ==, "X");
+
+	/* two chars */
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_LANGUAGE), ==, "in");
+
+	/* one word */
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_TITLE), ==, "POLL");
+
+	/* two words */
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_AUTHOR), ==, "POLL author");
+
+	/* long string */
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_DESCRIPTION), ==, description);
+
+	g_free (uri);
+}
+
 static void
 test_parsing_hadess (void)
 {
@@ -957,7 +1291,55 @@ test_parsing_m3u_streaming (void)
 	g_free (uri);
 }
 
-#ifdef HAVE_QUVI
+static void
+video_links_slow_parsing (const char *uri, gfloat timeout)
+{
+	time_t start, end;
+	double run_time;
+
+	g_setenv ("SLOW_PARSING", "1", TRUE);
+
+	start = time (NULL);
+	option_no_recurse = TRUE;
+	parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_TITLE);
+	end = time (NULL);
+
+	run_time = difftime (end, start);
+
+	g_assert_cmpfloat (run_time, <, timeout);
+}
+
+static void
+test_video_links_slow_parsing ()
+{
+	char *uri;
+	gfloat timeout = 2.0; /* seconds */
+
+	/* rss feed with 400 entries. should take 400 * 1 = 400
+	 * seconds with videosite check, and approx. less than 1
+	 * second if we bypass videosite check.
+	 */
+	uri = get_relative_uri (TEST_SRCDIR "podcast-different-item-images.rss");
+	video_links_slow_parsing (uri, timeout);
+	g_free (uri);
+
+	/* atom feed with 20 entries. should take 20 * 1 = 20
+	 * seconds with videosite check, and approx. less than 1
+	 * second if we bypass videosite check.
+	 */
+	uri = get_relative_uri (TEST_SRCDIR "gitlab-issues.atom");
+	video_links_slow_parsing (uri, timeout);
+	g_free (uri);
+
+	/* atom feed with 5 entries. should take 5 * 1 = 5
+	 * seconds with videosite check, and approx. less than 1
+	 * second if we bypass videosite check.
+	 */
+	uri = get_relative_uri (TEST_SRCDIR "status-gnome-org.atom");
+	video_links_slow_parsing (uri, timeout);
+	g_free (uri);
+}
+
 static void
 test_parsing_rss_id (void)
 {
@@ -975,7 +1357,6 @@ test_parsing_rss_link (void)
 	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, "http://www.guardian.co.uk/technology/audio/2011/may/03/tech-weekly-art-love-bin-laden");
 	g_free (uri);
 }
-#endif /* HAVE_QUVI */
 
 static void
 test_parsing_not_asx_playlist (void)
@@ -1019,6 +1400,16 @@ test_parsing_remote_mp3 (void)
 	g_assert_cmpint (simple_parser_test ("http://feeds.soundcloud.com/stream/303432626-opensourcesecuritypodcast-episode-28-rsa-conference-2017.mp3"), ==, TOTEM_PL_PARSER_RESULT_UNHANDLED);
 }
 
+static void
+test_xml_trailing_space (void)
+{
+	g_autofree char *uri = NULL;
+	g_test_bug ("28");
+	uri = get_relative_uri (TEST_SRCDIR "xml-trailing-space.xml");
+	/* URL from https://gitlab.gnome.org/GNOME/totem-pl-parser/-/issues/28 */
+	g_assert_cmpint (simple_parser_test (uri), ==, TOTEM_PL_PARSER_RESULT_SUCCESS);
+}
+
 static void
 test_parsing_not_really_php_but_html_instead (void)
 {
@@ -1188,6 +1579,66 @@ test_saving_async (void)
 	g_main_loop_run (loop);
 }
 
+static void
+test_saving_parsing_xspf_title (void)
+{
+	g_autoptr(TotemPlParser) parser = NULL;
+	g_autoptr(TotemPlPlaylist) playlist = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GFile) output = NULL;
+	TotemPlPlaylistIter pl_iter;
+	g_autofree char *path = NULL;
+	int fd;
+	char *uri;
+	char *title;
+
+	parser = totem_pl_parser_new ();
+	playlist = totem_pl_playlist_new ();
+	fd = g_file_open_tmp (NULL, &path, &error);
+	g_assert_no_error (error);
+	close (fd);
+	output = g_file_new_for_path (path);
+	uri = g_file_get_uri (output);
+
+	/* save empty playlist file and check empty title */
+	{
+		totem_pl_parser_save (parser,
+				      playlist,
+				      output,
+				      NULL, TOTEM_PL_PARSER_XSPF, &error);
+		g_assert_error (error, TOTEM_PL_PARSER_ERROR, TOTEM_PL_PARSER_ERROR_EMPTY_PLAYLIST);
+		g_clear_error (&error);
+		title = parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_TITLE);
+		g_assert_cmpstr (title, ==, NULL);
+	}
+
+	/* save non-empty playlist file with empty title and check empty title */
+	{
+		totem_pl_playlist_append (playlist, &pl_iter);
+		add_pl_iter_metadata (playlist, &pl_iter);
+		totem_pl_parser_save (parser,
+				      playlist,
+				      output,
+				      NULL, TOTEM_PL_PARSER_XSPF, &error);
+		g_assert_no_error (error);
+		title = parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_TITLE);
+		g_assert_cmpstr (title, ==, NULL);
+	}
+
+	/* save non-empty playlist file with non-empty title and check non-empty title */
+	{
+		totem_pl_parser_save (parser,
+				      playlist,
+				      output,
+				      "샘플 재생 목록 제목", TOTEM_PL_PARSER_XSPF, &error);
+		g_assert_no_error (error);
+		title = parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_TITLE);
+		g_assert_cmpstr (title, ==, "샘플 재생 목록 제목");
+	}
+
+	g_free (uri);
+}
+
 #define MAX_DESCRIPTION_LEN 128
 #define DATE_BUFSIZE 512
 #define PRINT_DATE_FORMAT "%Y-%m-%dT%H:%M:%SZ"
@@ -1403,6 +1854,9 @@ main (int argc, char *argv[])
 	/* If we've been given no URIs, run the static tests */
 	if (uris == NULL) {
 		check_http ();
+		g_setenv ("TOTEM_PL_PARSER_VIDEOSITE_SCRIPT", TEST_SRCDIR "/videosite-tester.sh", TRUE);
+
+		option_debug = TRUE;
 
 		g_test_add_func ("/parser/duration", test_duration);
 		g_test_add_func ("/parser/date", test_date);
@@ -1429,23 +1883,35 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/xml_comment_whitespace", test_parsing_xml_comment_whitespace);
 		g_test_add_func ("/parser/parsing/multi_line_rtsptext", test_parsing_rtsp_text_multi);
 		g_test_add_func ("/parser/parsing/single_line_rtsptext", test_parsing_rtsp_text);
-		g_test_add_func ("/parser/parsing/podcast_content_type", test_parsing_content_type);
+		g_test_add_func ("/parser/parsing/podcast_needle_carriage_return", test_parsing_needle_carriage_return);
+		g_test_add_func ("/parser/parsing/podcast_feed_content_type", test_parsing_feed_content_type);
+		g_test_add_func ("/parser/parsing/podcast_item_content_type", test_parsing_item_content_type);
 		g_test_add_func ("/parser/parsing/podcast_medium", test_parsing_medium);
+		g_test_add_func ("/parser/parsing/podcast_feed_description", test_parsing_feed_description);
+		g_test_add_func ("/parser/parsing/podcast_item_description", test_parsing_item_description);
+		g_test_add_func ("/parser/parsing/podcast_feed_image", test_parsing_feed_image);
+		g_test_add_func ("/parser/parsing/podcast_item_image", test_parsing_item_image);
+		g_test_add_func ("/parser/parsing/podcast_feed_pubdate", test_parsing_feed_pubdate);
+		g_test_add_func ("/parser/parsing/podcast_feed_author", test_parsing_feed_author);
+		g_test_add_func ("/parser/parsing/podcast_feed_explicit", test_parsing_feed_explicit);
+		g_test_add_func ("/parser/parsing/podcast_item_explicit", test_parsing_item_explicit);
+		g_test_add_func ("/parser/parsing/invalid_characters", test_invalid_characters);
+		g_test_add_func ("/parser/parsing/invalid_utf8_characters", test_invalid_utf8_characters);
 		g_test_add_func ("/parser/parsing/live_streaming", test_parsing_live_streaming);
 		g_test_add_func ("/parser/parsing/xml_mixed_cdata", test_parsing_xml_mixed_cdata);
 		g_test_add_func ("/parser/parsing/m3u_streaming", test_parsing_m3u_streaming);
-#ifdef HAVE_QUVI
+		g_test_add_func ("/parser/parsing/xml_cdata", test_parsing_xml_cdata);
 		g_test_add_func ("/parser/videosite", test_videosite);
 		g_test_add_func ("/parser/parsing/rss_id", test_parsing_rss_id);
 		g_test_add_func ("/parser/parsing/rss_link", test_parsing_rss_link);
 		g_test_add_func ("/parser/parsing/youtube_starttime", test_youtube_starttime);
-#endif /* HAVE_QUVI */
 		g_test_add_func ("/parser/parsing/not_asx_playlist", test_parsing_not_asx_playlist);
 		g_test_add_func ("/parser/parsing/not_really_php", test_parsing_not_really_php);
 		g_test_add_func ("/parser/parsing/not_really_php_but_html_instead", test_parsing_not_really_php_but_html_instead);
 		g_test_add_func ("/parser/parsing/num_items_in_pls", test_parsing_num_entries);
 		g_test_add_func ("/parser/parsing/xspf_genre", test_parsing_xspf_genre);
 		g_test_add_func ("/parser/parsing/xspf_escaping", test_parsing_xspf_escaping);
+		g_test_add_func ("/parser/parsing/xspf_metadata", test_parsing_xspf_metadata);
 		g_test_add_func ("/parser/parsing/xspf_xml_base", test_parsing_xspf_xml_base);
 		g_test_add_func ("/parser/parsing/test_pl_content_type", test_pl_content_type);
 		g_test_add_func ("/parser/parsing/itms_link", test_itms_parsing);
@@ -1459,8 +1925,12 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/async_signal_order", test_async_parsing_signal_order);
 		g_test_add_func ("/parser/parsing/wma_asf", test_parsing_wma_asf);
 		g_test_add_func ("/parser/parsing/remote_mp3", test_parsing_remote_mp3);
+		g_test_add_func ("/parser/parsing/xml_trailing_space", test_xml_trailing_space);
+		g_test_add_func ("/parser/saving/parsing/xspf_title", test_saving_parsing_xspf_title);
 		g_test_add_func ("/parser/saving/sync", test_saving_sync);
 		g_test_add_func ("/parser/saving/async", test_saving_async);
+		/* set an envvar, keep at the end */
+		g_test_add_func ("/parser/parsing/video_links_slow_parsing", test_video_links_slow_parsing);
 
 		return g_test_run ();
 	}
