@@ -35,53 +35,51 @@
 #include "totem-pl-parser-videosite.h"
 #include "totem-pl-parser-private.h"
 
-#define RSS_NEEDLE "<rss "
-#define RSS_NEEDLE2 "<rss\n"
-#define ATOM_NEEDLE "<feed "
-#define OPML_NEEDLE "<opml "
+#define RSS_NEEDLE "<rss"
+#define ATOM_NEEDLE "<feed"
+#define OPML_NEEDLE "<opml"
+
+static const char *
+totem_pl_parser_is_xml_type (const char *data,
+			     gsize len,
+			     const char *needle,
+			     const char *mimetype)
+{
+	gchar *found;
+	gchar separator;
+
+	g_return_val_if_fail (len > 0, NULL);
+
+	if (len > MIME_READ_CHUNK_SIZE)
+		len = MIME_READ_CHUNK_SIZE;
+
+	found = g_strstr_len (data, len, needle);
+	if (!found)
+		return NULL;
+
+	separator = *(found + strlen(needle));
+	if (g_ascii_isspace (separator))
+		return mimetype;
+
+	return NULL;
+}
 
 const char *
 totem_pl_parser_is_rss (const char *data, gsize len)
 {
-	if (len == 0)
-		return FALSE;
-	if (len > MIME_READ_CHUNK_SIZE)
-		len = MIME_READ_CHUNK_SIZE;
-
-	if (g_strstr_len (data, len, RSS_NEEDLE) != NULL)
-		return RSS_MIME_TYPE;
-	if (g_strstr_len (data, len, RSS_NEEDLE2) != NULL)
-		return RSS_MIME_TYPE;
-
-	return NULL;
+	return totem_pl_parser_is_xml_type (data, len, RSS_NEEDLE, RSS_MIME_TYPE);
 }
 
 const char *
 totem_pl_parser_is_atom (const char *data, gsize len)
 {
-	if (len == 0)
-		return FALSE;
-	if (len > MIME_READ_CHUNK_SIZE)
-		len = MIME_READ_CHUNK_SIZE;
-
-	if (g_strstr_len (data, len, ATOM_NEEDLE) != NULL)
-		return ATOM_MIME_TYPE;
-
-	return NULL;
+	return totem_pl_parser_is_xml_type (data, len, ATOM_NEEDLE, ATOM_MIME_TYPE);
 }
 
 const char *
 totem_pl_parser_is_opml (const char *data, gsize len)
 {
-	if (len == 0)
-		return FALSE;
-	if (len > MIME_READ_CHUNK_SIZE)
-		len = MIME_READ_CHUNK_SIZE;
-
-	if (g_strstr_len (data, len, OPML_NEEDLE) != NULL)
-		return OPML_MIME_TYPE;
-
-	return NULL;
+	return totem_pl_parser_is_xml_type (data, len, OPML_NEEDLE, OPML_MIME_TYPE);
 }
 
 const char *
@@ -98,6 +96,21 @@ totem_pl_parser_is_xml_feed (const char *data, gsize len)
 
 #ifndef TOTEM_PL_PARSER_MINI
 
+static const char *
+xml_parser_get_node_value (xml_node_t *parent, const char *node_name)
+{
+	xml_node_t *child;
+
+	for (child = parent->child; child != NULL; child = child->next) {
+		if (child->name == NULL)
+			continue;
+		if (g_ascii_strcasecmp (child->name, node_name) == 0)
+			return child->data;
+	}
+
+	return NULL;
+}
+
 static gboolean
 is_image (const char *url)
 {
@@ -113,15 +126,75 @@ is_image (const char *url)
 	return retval;
 }
 
+static void
+set_longer_description (xml_node_t *node, const char **description)
+{
+	if (node->data == NULL)
+		return;
+	if (*description) {
+		/* prefer longer descriptions */
+		if (strlen (node->data) > strlen (*description))
+			*description = node->data;
+	} else {
+		*description = node->data;
+	}
+}
+
+static void
+set_recent_date (xml_node_t *node, const char **date)
+{
+	if (node->data == NULL)
+		return;
+
+	if (*date) {
+		guint64 old, new;
+
+		old = totem_pl_parser_parse_date (*date, FALSE);
+		new = totem_pl_parser_parse_date (node->data, FALSE);
+
+		/* prefer recent date */
+		if (new <= old)
+			return;
+	}
+
+	*date = node->data;
+}
+
+static const char*
+get_content_rating (const char *value)
+{
+	gchar *str;
+	const gchar* content_rating = TOTEM_PL_PARSER_CONTENT_RATING_UNRATED;
+
+	if (value) {
+		str = g_strdup (value);
+		g_strstrip (str);
+
+		if (g_ascii_strcasecmp (str, "no") == 0 ||
+		    g_ascii_strcasecmp (str, "false") == 0 ||
+		    g_ascii_strcasecmp (str, "clean") == 0) {
+			content_rating = TOTEM_PL_PARSER_CONTENT_RATING_CLEAN;
+		} else if (g_ascii_strcasecmp (str, "yes") == 0 ||
+			   g_ascii_strcasecmp (str, "true") == 0 ||
+			   g_ascii_strcasecmp (str, "explicit") == 0) {
+			content_rating = TOTEM_PL_PARSER_CONTENT_RATING_EXPLICIT;
+		}
+
+		g_free (str);
+	}
+
+	return content_rating;
+}
+
 static TotemPlParserResult
 parse_rss_item (TotemPlParser *parser, xml_node_t *parent)
 {
-	const char *title, *uri, *description, *author, *img;
+	const char *title, *uri, *description, *author, *img, *explicit;
 	const char *pub_date, *duration, *filesize, *content_type, *id;
 	xml_node_t *node;
 
 	title = uri = description = author = content_type = NULL;
-	img = pub_date = duration = filesize = id = NULL;
+	img = pub_date = duration = filesize = id = explicit = NULL;
 
 	for (node = parent->child; node != NULL; node = node->next) {
 		if (node->name == NULL)
@@ -137,7 +210,8 @@ parse_rss_item (TotemPlParser *parser, xml_node_t *parent)
 			id = node->data;
 		} else if (g_ascii_strcasecmp (node->name, "description") == 0
 			   || g_ascii_strcasecmp (node->name, "itunes:summary") == 0) {
-			description = node->data;
+			/* prefer longer item descriptions */
+			set_longer_description (node, &description);
 		} else if (g_ascii_strcasecmp (node->name, "author") == 0
 			   || g_ascii_strcasecmp (node->name, "itunes:author") == 0) {
 			author = node->data;
@@ -150,18 +224,25 @@ parse_rss_item (TotemPlParser *parser, xml_node_t *parent)
 
 			tmp = xml_parser_get_property (node, "medium");
 			if (tmp != NULL && g_str_equal (tmp, "image")) {
-				img = xml_parser_get_property (node, "url");
+				tmp = xml_parser_get_property (node, "url");
+				if (tmp != NULL)
+					img = tmp;
 				continue;
 			}
 
 			tmp = xml_parser_get_property (node, "type");
 			if (tmp != NULL &&
 			    g_str_has_prefix (tmp, "audio/") == FALSE) {
-				if (g_str_has_prefix (tmp, "image/"))
-					img = xml_parser_get_property (node, "url");
+				if (g_str_has_prefix (tmp, "image/")) {
+					tmp = xml_parser_get_property (node, "url");
+					if (tmp != NULL)
+						img = tmp;
+				}
 				continue;
 			}
-			content_type = tmp;
+
+			if (tmp != NULL)
+				content_type = tmp;
 
 			tmp = xml_parser_get_property (node, "url");
 			if (tmp != NULL)
@@ -187,15 +268,35 @@ parse_rss_item (TotemPlParser *parser, xml_node_t *parent)
 			tmp = xml_parser_get_property (node, "length");
 			if (tmp != NULL)
 				filesize = tmp;
+
+			tmp = xml_parser_get_property (node, "type");
+			if (tmp != NULL)
+				content_type = tmp;
 		} else if (g_ascii_strcasecmp (node->name, "link") == 0 &&
-			   totem_pl_parser_is_videosite (node->data, FALSE) != FALSE) {
+			   totem_pl_parser_get_recurse (parser) &&
+			   totem_pl_parser_is_videosite (node->data, totem_pl_parser_is_debugging_enabled (parser)) != FALSE) {
 			uri = node->data;
+		} else if (g_ascii_strcasecmp (node->name, "image") == 0) {
+			const char *tmp;
+
+			tmp = xml_parser_get_node_value (node, "url");
+			if (tmp != NULL)
+				img = tmp;
+		} else if (g_ascii_strcasecmp (node->name, "itunes:image") == 0) {
+			const char *tmp;
+
+			tmp = xml_parser_get_property (node, "href");
+			if (tmp != NULL)
+				img = tmp;
+		} else if (g_ascii_strcasecmp (node->name, "itunes:explicit") == 0) {
+			explicit = node->data;
 		}
 	}
 
 	if (id != NULL &&
 	    uri == NULL &&
-	    totem_pl_parser_is_videosite (id, FALSE) != FALSE)
+	    totem_pl_parser_get_recurse (parser) &&
+	    totem_pl_parser_is_videosite (id, totem_pl_parser_is_debugging_enabled (parser)) != FALSE)
 		uri = id;
 
 	if (uri != NULL) {
@@ -210,6 +311,7 @@ parse_rss_item (TotemPlParser *parser, xml_node_t *parent)
 					 TOTEM_PL_PARSER_FIELD_FILESIZE, filesize,
 					 TOTEM_PL_PARSER_FIELD_CONTENT_TYPE, content_type,
 					 TOTEM_PL_PARSER_FIELD_IMAGE_URI, img,
+					 TOTEM_PL_PARSER_FIELD_CONTENT_RATING, get_content_rating (explicit),
 					 NULL);
 	}
 
@@ -220,11 +322,11 @@ static TotemPlParserResult
 parse_rss_items (TotemPlParser *parser, const char *uri, xml_node_t *parent)
 {
 	const char *title, *language, *description, *author;
-	const char *contact, *img, *pub_date, *copyright;
+	const char *contact, *img, *pub_date, *copyright, *generator, *explicit;
 	xml_node_t *node;
 
 	title = language = description = author = NULL;
-	contact = img = pub_date = copyright = NULL;
+	contact = img = pub_date = copyright = generator = explicit = NULL;
 
 	/* We need to parse for the feed metadata first, then for the items */
 	for (node = parent->child; node != NULL; node = node->next) {
@@ -236,16 +338,32 @@ parse_rss_items (TotemPlParser *parser, const char *uri, xml_node_t *parent)
 		} else if (g_ascii_strcasecmp (node->name, "language") == 0) {
 			language = node->data;
 		} else if (g_ascii_strcasecmp (node->name, "description") == 0
-			 || g_ascii_strcasecmp (node->name, "itunes:subtitle") == 0) {
-		    	description = node->data;
+			 || g_ascii_strcasecmp (node->name, "itunes:subtitle") == 0
+			 || g_ascii_strcasecmp (node->name, "itunes:summary") == 0) {
+			/* prefer longer feed descriptions */
+			set_longer_description (node, &description);
 		} else if (g_ascii_strcasecmp (node->name, "author") == 0
-			 || g_ascii_strcasecmp (node->name, "itunes:author") == 0
-			 || (g_ascii_strcasecmp (node->name, "generator") == 0 && author == NULL)) {
-		    	author = node->data;
+			   || g_ascii_strcasecmp (node->name, "itunes:author") == 0) {
+			if (node->data)
+				author = node->data;
+		} else if (g_ascii_strcasecmp (node->name, "generator") == 0) {
+			generator = node->data;
+		} else if (g_ascii_strcasecmp (node->name, "itunes:owner") == 0) {
+			const char *tmp;
+
+			/* Owner name is much broader than author. So, we set it
+			 * only if there is no author in feed yet. */
+			tmp = xml_parser_get_node_value (node, "itunes:name");
+			if (tmp != NULL && author == NULL)
+				author = tmp;
 		} else if (g_ascii_strcasecmp (node->name, "webMaster") == 0) {
 			contact = node->data;
 		} else if (g_ascii_strcasecmp (node->name, "image") == 0) {
-			img = node->data;
+			const char *tmp;
+
+			tmp = xml_parser_get_node_value (node, "url");
+			if (tmp != NULL)
+				img = tmp;
 		} else if (g_ascii_strcasecmp (node->name, "itunes:image") == 0) {
 			const char *href;
 
@@ -253,13 +371,20 @@ parse_rss_items (TotemPlParser *parser, const char *uri, xml_node_t *parent)
 			if (href != NULL)
 				img = href;
 		} else if (g_ascii_strcasecmp (node->name, "lastBuildDate") == 0
-			 || g_ascii_strcasecmp (node->name, "pubDate") == 0) {
-		    	pub_date = node->data;
+			   || (g_ascii_strcasecmp (node->name, "pubDate") == 0)) {
+			/* prefer recent of <lastBuildDate> and <pubDate> date */
+			set_recent_date (node, &pub_date);
 		} else if (g_ascii_strcasecmp (node->name, "copyright") == 0) {
 			copyright = node->data;
+		} else if (g_ascii_strcasecmp (node->name, "itunes:explicit") == 0) {
+			explicit = node->data;
 		}
 	}
 
+	/* update generator as author, only as last resort */
+	if (!author && generator)
+		author = generator;
+
 	/* Send the info we already have about the feed */
 	totem_pl_parser_add_uri (parser,
 				 TOTEM_PL_PARSER_FIELD_IS_PLAYLIST, TRUE,
@@ -272,6 +397,8 @@ parse_rss_items (TotemPlParser *parser, const char *uri, xml_node_t *parent)
 				 TOTEM_PL_PARSER_FIELD_COPYRIGHT, copyright,
 				 TOTEM_PL_PARSER_FIELD_IMAGE_URI, img,
 				 TOTEM_PL_PARSER_FIELD_CONTACT, contact,
+				 TOTEM_PL_PARSER_FIELD_CONTENT_RATING, get_content_rating (explicit),
+				 TOTEM_PL_PARSER_FIELD_CONTENT_TYPE, "application/rss+xml",
 				 NULL);
 
 	for (node = parent->child; node != NULL; node = node->next) {
@@ -309,8 +436,7 @@ totem_pl_parser_add_rss (TotemPlParser *parser,
 
 	/* If the document has no name */
 	if (doc->name == NULL
-	    || (g_ascii_strcasecmp (doc->name , "rss") != 0
-		&& g_ascii_strcasecmp (doc->name , "rss\n") != 0)) {
+	    || (g_ascii_strcasecmp (doc->name , "rss") != 0)) {
 		g_free (contents);
 		xml_parser_free_tree (doc);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
@@ -419,22 +545,20 @@ parse_atom_entry (TotemPlParser *parser, xml_node_t *parent)
 		if (g_ascii_strcasecmp (node->name, "title") == 0) {
 			title = node->data;
 		} else if (g_ascii_strcasecmp (node->name, "author") == 0) {
-			xml_node_t *child;
+			const char *tmp;
 
-			for (child = node->child; child != NULL; child = child->next) {
-				if (child->name == NULL)
-					continue;
-
-				if (g_ascii_strcasecmp (child->name, "name") == 0) {
-					author = child->data;
-					break;
-				}
-			}
+			tmp = xml_parser_get_node_value (node, "name");
+			if (tmp != NULL)
+				author = tmp;
 		} else if (g_ascii_strcasecmp (node->name, "link") == 0) {
 			const char *rel;
 
 			//FIXME how do we choose the default enclosure type?
 			rel = xml_parser_get_property (node, "rel");
+
+			if (rel == NULL)
+				continue;
+
 			if (g_ascii_strcasecmp (rel, "enclosure") == 0) {
 				const char *href;
 
@@ -458,8 +582,10 @@ parse_atom_entry (TotemPlParser *parser, xml_node_t *parent)
 				href = xml_parser_get_property (node, "href");
 				if (href == NULL)
 					continue;
-				if (!totem_pl_parser_is_videosite (href, FALSE))
+				if (totem_pl_parser_get_recurse (parser) &&
+				    !totem_pl_parser_is_videosite (href, totem_pl_parser_is_debugging_enabled (parser))) {
 					continue;
+				}
 				uri = href;
 			}
 		} else if (g_ascii_strcasecmp (node->name, "updated") == 0
@@ -492,8 +618,10 @@ parse_atom_entry (TotemPlParser *parser, xml_node_t *parent)
 					prop = xml_parser_get_property (child, "url");
 					if (prop == NULL)
 						continue;
-					if (!totem_pl_parser_is_videosite (prop, FALSE))
+					if (totem_pl_parser_get_recurse (parser) &&
+					    !totem_pl_parser_is_videosite (prop, totem_pl_parser_is_debugging_enabled (parser))) {
 						continue;
+					}
 					uri = prop;
 				} else if (g_ascii_strcasecmp (child->name, "media:thumbnail") == 0) {
 					img = xml_parser_get_property (child, "url");
@@ -560,6 +688,7 @@ parse_atom_entries (TotemPlParser *parser, const char *uri, xml_node_t *parent)
 							 TOTEM_PL_PARSER_FIELD_AUTHOR, author,
 							 TOTEM_PL_PARSER_FIELD_PUB_DATE, pub_date,
 							 TOTEM_PL_PARSER_FIELD_IMAGE_URI, img,
+							 TOTEM_PL_PARSER_FIELD_CONTENT_TYPE, "application/atom+xml",
 							 NULL);
 				started = TRUE;
 			}
@@ -640,6 +769,9 @@ totem_pl_parser_parse_json (char *data, gsize len, gboolean debug)
 {
 	char *s, *end;
 
+	if (debug)
+		g_print ("About to parse JSON:\n%s", data);
+
 	s = g_strstr_len (data, len, "feedUrl\":\"");
 	if (s == NULL)
 		return NULL;
@@ -699,7 +831,7 @@ totem_pl_parser_add_itms (TotemPlParser *parser,
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
 
-	DEBUG(file, g_print ("Got ID '%s' for URL '%s'", id, uri));
+	DEBUG(file, g_print ("Got ID '%s' for URL '%s'\n", id, uri));
 
 	json_uri = g_strdup_printf ("https://itunes.apple.com/lookup?id=%s&entity=podcast", id);
 	g_free (id);
@@ -801,6 +933,7 @@ parse_opml_head_body (TotemPlParser *parser, const char *uri, xml_node_t *parent
 				totem_pl_parser_add_uri (parser,
 							 TOTEM_PL_PARSER_FIELD_IS_PLAYLIST, TRUE,
 							 TOTEM_PL_PARSER_FIELD_URI, uri,
+							 TOTEM_PL_PARSER_FIELD_CONTENT_TYPE, "text/x-opml+xml",
 							 NULL);
 				started = TRUE;
 			}
