@@ -132,6 +132,9 @@
 
 #ifndef TOTEM_PL_PARSER_MINI
 #include <gobject/gvaluecollector.h>
+#ifdef HAVE_UCHARDET
+#include <uchardet.h>
+#endif
 
 #include "totem-pl-parser.h"
 #include "totemplparser-marshal.h"
@@ -156,6 +159,7 @@
 
 #define READ_CHUNK_SIZE 8192
 #define RECURSE_LEVEL_MAX 4
+#define ILLEGAL_CONTEXT_LENGTH 20
 
 #define D(x) if (debug) x
 
@@ -573,6 +577,10 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 				     "The default audio-track to play", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("content-rating", "content-rating",
+				     "String representing the content rating for an entry", TOTEM_PL_PARSER_CONTENT_RATING_UNRATED,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 }
 
 static void
@@ -803,6 +811,12 @@ totem_pl_parser_is_debugging_enabled (TotemPlParser *parser)
 	return parser->priv->debug;
 }
 
+gboolean
+totem_pl_parser_get_recurse (TotemPlParser *parser)
+{
+	return parser->priv->recurse;
+}
+
 /**
  * totem_pl_parser_base_uri:
  * @uri: a URI
@@ -1782,8 +1796,8 @@ totem_pl_parser_glob_is_ignored (TotemPlParser *parser,
 gboolean
 totem_pl_parser_ignore (TotemPlParser *parser, const char *uri)
 {
-	g_autofree char *mimetype;
-	g_autoptr(GFile) file;
+	g_autofree char *mimetype = NULL;
+	g_autoptr(GFile) file = NULL;
 	guint i;
 
 	if (totem_pl_parser_glob_is_ignored (parser, uri) != FALSE)
@@ -1842,13 +1856,43 @@ totem_pl_parser_cleanup_xml (char *contents)
 	}
 }
 
+#ifdef HAVE_UCHARDET
+static char *
+guess_text_encoding (const char *text,
+		     gsize       len)
+{
+	uchardet_t handle;
+	char *encoding = NULL;
+	int ret;
+
+	handle = uchardet_new ();
+	ret = uchardet_handle_data (handle, text, len);
+	if (ret == 0) {
+		uchardet_data_end (handle);
+		encoding = g_strdup (uchardet_get_charset (handle));
+	}
+
+	uchardet_delete (handle);
+	return encoding;
+}
+#else
+static char *
+guess_text_encoding (const char *text,
+		     gsize       len)
+{
+	return NULL;
+}
+#endif /* HAVE_UCHARDET */
+
 xml_node_t *
 totem_pl_parser_parse_xml_relaxed (char *contents,
 				   gsize size)
 {
 	xml_node_t* doc, *node;
-	char *encoding, *new_contents;
-	gsize new_size;
+	g_autoptr(GError) error = NULL;
+	g_autofree char *encoding = NULL;
+	g_autofree char *new_contents = NULL;
+	gsize new_size, bytes_read;
 	xml_parser_t *xml_parser;
 
 	totem_pl_parser_cleanup_xml (contents);
@@ -1868,30 +1912,49 @@ totem_pl_parser_parse_xml_relaxed (char *contents,
 		break;
 	}
 
-	if (encoding == NULL || g_str_equal (encoding, "UTF-8") != FALSE) {
+	if (encoding == NULL || g_ascii_strcasecmp (encoding, "UTF-8") == 0) {
+		if (g_utf8_validate (contents, -1, NULL))
+			return doc;
+		g_debug ("Document %s pretended to be in UTF-8 but didn't validate",
+			 encoding ? "explicitly" : "implicitly");
 		g_free (encoding);
-		return doc;
+		encoding = guess_text_encoding (contents, size);
+		if (!encoding)
+			return NULL;
+
+		g_debug ("Guessed text encoding of XML data as '%s'", encoding);
+		/* fall-through with the detected encoding */
 	}
 
 	xml_parser_free_tree (doc);
 
-	new_contents = g_convert (contents, size, "UTF-8", encoding, NULL, &new_size, NULL);
+	new_contents = g_convert (contents, size, "UTF-8", encoding, &bytes_read, &new_size, &error);
 	if (new_contents == NULL) {
-		g_warning ("Failed to convert XML data to UTF-8");
-		g_free (encoding);
+		g_autofree char *message = NULL;
+		message = g_strdup_printf ("Failed to convert XML data from '%s' to '%s': %s",
+					   encoding, "UTF-8", error->message);
+
+		if (error->code == G_CONVERT_ERROR_ILLEGAL_SEQUENCE) {
+			int context_length = MIN (bytes_read, ILLEGAL_CONTEXT_LENGTH);
+
+			g_warning ("%s: byte offset %" G_GSIZE_FORMAT ", byte: '%.1s', byte context: '%.*s'",
+				   message, bytes_read, contents + bytes_read,
+				   context_length + 1,
+				   contents + bytes_read - context_length);
+		} else {
+			g_warning ("%s", message);
+		}
+
 		return NULL;
 	}
-	g_free (encoding);
 
 	xml_parser = xml_parser_init_r (new_contents, new_size, XML_PARSER_CASE_INSENSITIVE);
 	if (xml_parser_build_tree_with_options_r (xml_parser, &doc, XML_PARSER_RELAXED | XML_PARSER_MULTI_TEXT) < 0) {
 		xml_parser_finalize_r (xml_parser);
-		g_free (new_contents);
 		return NULL;
 	}
 
 	xml_parser_finalize_r (xml_parser);
-	g_free (new_contents);
 
 	return doc;
 }
@@ -1996,7 +2059,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 	/* Should we try to parse it with quvi? */
 	if (g_file_has_uri_scheme (file, "http") ||
 	    g_file_has_uri_scheme (file, "https")) {
-		if (uri != NULL && totem_pl_parser_is_videosite (uri, parser->priv->debug) != FALSE) {
+		if (uri != NULL && parse_data->recurse && totem_pl_parser_is_videosite (uri, parser->priv->debug) != FALSE) {
 			ret = totem_pl_parser_add_videosite (parser, file, base_file, parse_data, NULL);
 			if (ret == TOTEM_PL_PARSER_RESULT_SUCCESS)
 				return ret;
