@@ -51,13 +51,10 @@ struct TotemPlPlaylistPrivate {
         GList *items;
 };
 
-#define TOTEM_PL_PLAYLIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TOTEM_TYPE_PL_PLAYLIST, TotemPlPlaylistPrivate))
-
 static void totem_pl_playlist_finalize (GObject *object);
 
-
-G_DEFINE_TYPE (TotemPlPlaylist, totem_pl_playlist, G_TYPE_OBJECT)
-
+G_DEFINE_TYPE_WITH_CODE (TotemPlPlaylist, totem_pl_playlist, G_TYPE_OBJECT,
+			 G_ADD_PRIVATE (TotemPlPlaylist))
 
 static void
 totem_pl_playlist_class_init (TotemPlPlaylistClass *klass)
@@ -65,8 +62,6 @@ totem_pl_playlist_class_init (TotemPlPlaylistClass *klass)
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
         object_class->finalize = totem_pl_playlist_finalize;
-
-        g_type_class_add_private (klass, sizeof (TotemPlPlaylistPrivate));
 }
 
 static void
@@ -79,7 +74,7 @@ totem_pl_playlist_finalize (GObject *object)
 {
         TotemPlPlaylistPrivate *priv;
 
-        priv = TOTEM_PL_PLAYLIST_GET_PRIVATE (object);
+        priv = totem_pl_playlist_get_instance_private (TOTEM_PL_PLAYLIST (object));
 
         g_list_foreach (priv->items, (GFunc) g_hash_table_destroy, NULL);
         g_list_free (priv->items);
@@ -113,9 +108,9 @@ totem_pl_playlist_size (TotemPlPlaylist *playlist)
 {
         TotemPlPlaylistPrivate *priv;
 
-        g_return_val_if_fail (TOTEM_IS_PL_PLAYLIST (playlist), 0);
+        g_return_val_if_fail (TOTEM_PL_IS_PLAYLIST (playlist), 0);
 
-        priv = TOTEM_PL_PLAYLIST_GET_PRIVATE (playlist);
+        priv = totem_pl_playlist_get_instance_private (playlist);
 
         return g_list_length (priv->items);
 }
@@ -145,10 +140,10 @@ totem_pl_playlist_prepend (TotemPlPlaylist     *playlist,
         TotemPlPlaylistPrivate *priv;
         GHashTable *item;
 
-        g_return_if_fail (TOTEM_IS_PL_PLAYLIST (playlist));
+        g_return_if_fail (TOTEM_PL_IS_PLAYLIST (playlist));
         g_return_if_fail (iter != NULL);
 
-        priv = TOTEM_PL_PLAYLIST_GET_PRIVATE (playlist);
+        priv = totem_pl_playlist_get_instance_private (playlist);
 
         item = create_playlist_item ();
         priv->items = g_list_prepend (priv->items, item);
@@ -174,10 +169,10 @@ totem_pl_playlist_append (TotemPlPlaylist     *playlist,
         GHashTable *item;
         GList *list_item;
 
-        g_return_if_fail (TOTEM_IS_PL_PLAYLIST (playlist));
+        g_return_if_fail (TOTEM_PL_IS_PLAYLIST (playlist));
         g_return_if_fail (iter != NULL);
 
-        priv = TOTEM_PL_PLAYLIST_GET_PRIVATE (playlist);
+        priv = totem_pl_playlist_get_instance_private (playlist);
 
         item = create_playlist_item ();
 
@@ -211,10 +206,10 @@ totem_pl_playlist_insert (TotemPlPlaylist     *playlist,
         TotemPlPlaylistPrivate *priv;
         GHashTable *item;
 
-        g_return_if_fail (TOTEM_IS_PL_PLAYLIST (playlist));
+        g_return_if_fail (TOTEM_PL_IS_PLAYLIST (playlist));
         g_return_if_fail (iter != NULL);
 
-        priv = TOTEM_PL_PLAYLIST_GET_PRIVATE (playlist);
+        priv = totem_pl_playlist_get_instance_private (playlist);
 
         item = create_playlist_item ();
         priv->items = g_list_insert (priv->items, item, position);
@@ -237,7 +232,7 @@ check_iter (TotemPlPlaylist     *playlist,
                 return FALSE;
         }
 
-        priv = TOTEM_PL_PLAYLIST_GET_PRIVATE (playlist);
+        priv = totem_pl_playlist_get_instance_private (playlist);
 
         if (g_list_position (priv->items, iter->data2) == -1) {
                 return FALSE;
@@ -261,10 +256,10 @@ totem_pl_playlist_iter_first (TotemPlPlaylist     *playlist,
 {
         TotemPlPlaylistPrivate *priv;
 
-        g_return_val_if_fail (TOTEM_IS_PL_PLAYLIST (playlist), FALSE);
+        g_return_val_if_fail (TOTEM_PL_IS_PLAYLIST (playlist), FALSE);
         g_return_val_if_fail (iter != NULL, FALSE);
 
-        priv = TOTEM_PL_PLAYLIST_GET_PRIVATE (playlist);
+        priv = totem_pl_playlist_get_instance_private (playlist);
 
         if (!priv->items) {
                 /* Empty playlist */
@@ -293,7 +288,7 @@ gboolean
 totem_pl_playlist_iter_next (TotemPlPlaylist     *playlist,
                              TotemPlPlaylistIter *iter)
 {
-        g_return_val_if_fail (TOTEM_IS_PL_PLAYLIST (playlist), FALSE);
+        g_return_val_if_fail (TOTEM_PL_IS_PLAYLIST (playlist), FALSE);
         g_return_val_if_fail (check_iter (playlist, iter), FALSE);
 
         iter->data2 = ((GList *) iter->data2)->next;
@@ -317,7 +312,7 @@ gboolean
 totem_pl_playlist_iter_prev (TotemPlPlaylist     *playlist,
                              TotemPlPlaylistIter *iter)
 {
-        g_return_val_if_fail (TOTEM_IS_PL_PLAYLIST (playlist), FALSE);
+        g_return_val_if_fail (TOTEM_PL_IS_PLAYLIST (playlist), FALSE);
         g_return_val_if_fail (check_iter (playlist, iter), FALSE);
 
         iter->data2 = ((GList *) iter->data2)->prev;
@@ -346,7 +341,7 @@ totem_pl_playlist_get_value (TotemPlPlaylist     *playlist,
         GHashTable *item_data;
         gchar *str;
 
-        g_return_val_if_fail (TOTEM_IS_PL_PLAYLIST (playlist), FALSE);
+        g_return_val_if_fail (TOTEM_PL_IS_PLAYLIST (playlist), FALSE);
         g_return_val_if_fail (check_iter (playlist, iter), FALSE);
         g_return_val_if_fail (key != NULL, FALSE);
         g_return_val_if_fail (value != NULL, FALSE);
@@ -381,7 +376,7 @@ totem_pl_playlist_get_valist (TotemPlPlaylist     *playlist,
         GHashTable *item_data;
         gchar *key, **value;
 
-        g_return_if_fail (TOTEM_IS_PL_PLAYLIST (playlist));
+        g_return_if_fail (TOTEM_PL_IS_PLAYLIST (playlist));
         g_return_if_fail (check_iter (playlist, iter));
 
         item_data = ((GList *) iter->data2)->data;
@@ -418,7 +413,7 @@ totem_pl_playlist_get (TotemPlPlaylist     *playlist,
 {
         va_list args;
 
-        g_return_if_fail (TOTEM_IS_PL_PLAYLIST (playlist));
+        g_return_if_fail (TOTEM_PL_IS_PLAYLIST (playlist));
         g_return_if_fail (check_iter (playlist, iter));
 
         va_start (args, iter);
@@ -446,7 +441,7 @@ totem_pl_playlist_set_value (TotemPlPlaylist     *playlist,
         GHashTable *item_data;
         gchar *str;
 
-        g_return_val_if_fail (TOTEM_IS_PL_PLAYLIST (playlist), FALSE);
+        g_return_val_if_fail (TOTEM_PL_IS_PLAYLIST (playlist), FALSE);
         g_return_val_if_fail (check_iter (playlist, iter), FALSE);
         g_return_val_if_fail (key != NULL, FALSE);
         g_return_val_if_fail (value != NULL, FALSE);
@@ -495,7 +490,7 @@ totem_pl_playlist_set_valist (TotemPlPlaylist     *playlist,
         GHashTable *item_data;
         gchar *key, *value;
 
-        g_return_if_fail (TOTEM_IS_PL_PLAYLIST (playlist));
+        g_return_if_fail (TOTEM_PL_IS_PLAYLIST (playlist));
         g_return_if_fail (check_iter (playlist, iter));
 
         item_data = ((GList *) iter->data2)->data;
@@ -529,7 +524,7 @@ totem_pl_playlist_set (TotemPlPlaylist     *playlist,
 {
         va_list args;
 
-        g_return_if_fail (TOTEM_IS_PL_PLAYLIST (playlist));
+        g_return_if_fail (TOTEM_PL_IS_PLAYLIST (playlist));
         g_return_if_fail (check_iter (playlist, iter));
 
         va_start (args, iter);
